# save as encrypted so others can't reach it
from azure.storage.blob import BlobServiceClient
from cryptography.fernet import Fernet
import pickle
import pandas as pd

def upload_df_as_encrypted_pickle(
    df_to_upload: pd.DataFrame,
    connection_string: str, 
    container_name: str, 
    blob_name: str,
    fernet_key_string: str,
):
    '''
    Uploads to container: file will be unreadable without the fernet_key_string

    Useful if you don't have access to a secure container and need to enforce access control

    fernet_key_string must be a string of 32 characters

    You can make a key with:
    key = Fernet.generate_key()
    fernet_key_string = key.decode()
    '''

    fernet_key = bytes(fernet_key_string, 'UTF-8')
    fernet = Fernet(fernet_key)

    df_as_string = pickle.dumps(df_to_upload.to_dict())
    encoded_bytes = fernet.encrypt(df_as_string)

    blob_service_client = BlobServiceClient.from_connection_string(connection_string)
    container_client = blob_service_client.get_container_client(container_name)
    blob_client = container_client.get_blob_client(blob_name)

    # Upload data to the blob
    blob_client.upload_blob(encoded_bytes, overwrite = True)
    
    print(f'Uploaded file to {container_name}/{blob_name}')


connection_string = 'DefaultEndpointsProtocol=https;AccountName=hmtukshawkeyedatalakedev;AccountKey=lG8ER8LaGyUdvYDmt4cEwYRmChC+pD05O20RoJkKNFFxYJUGSAy7KXcVg+b0Y/cmF+qF/acBSYKi+AStpmbXDg==;EndpointSuffix=core.windows.net'
container_name = 'test'
blob_name = 'example.pickle'

upload_df_as_encrypted_pickle(
    department_code_to_forecasts_and_features[department_code],
    connection_string, 
    container_name, 
    blob_name,
    key_string,
)





from azure.storage.blob import BlobServiceClient
import json

def upload_string_to_blob(connection_string, container_name, blob_name, data):
    '''
    # get the connection_string in 'Storage Accounts -> (account name) -> Access keys
    # hawkeye one is here:
    # https://portal.azure.com/#@tris42.onmicrosoft.com/resource/subscriptions/0c712683-db62-4326-a898-b4c354818292/resourceGroups/hmt-uks-hawkeye-dev-rg/providers/Microsoft.Storage/storageAccounts/hmtukshawkeyedatalakedev/keys

    # Example of running:

    connection_string = "COPY ME FROM URL ABOVE"
    container_name = "ADD THIS"
    blob_name = "bloomberg_py.txt"
    string_to_upload = json.dumps({
        'some dictionary':'example'
    })

    upload_string_to_blob(connection_string, container_name, blob_name, data)
    '''

    # Create the BlobServiceClient object which will be used to create a container client
    blob_service_client = BlobServiceClient.from_connection_string(connection_string)
    
    # Get a reference to a container
    container_client = blob_service_client.get_container_client(container_name)

    # Create the container if it doesn't exist (Optional)
    container_client.create_container()

    # Convert the string data to bytes, for blob upload
    byte_data = data.encode('utf-8')

    # Get a blob client for our blob
    blob_client = container_client.get_blob_client(blob_name)

    # Upload data to the blob
    blob_client.upload_blob(byte_data)





from azure.storage.blob import BlobServiceClient
import json


def download_blob_to_string(connection_string, container_name, blob_name):
    # Create the BlobServiceClient object which will be used to create a container client
    blob_service_client = BlobServiceClient.from_connection_string(connection_string)
    
    # Get a reference to a container
    container_client = blob_service_client.get_container_client(container_name)
    
    # Get a blob client for our blob
    blob_client = container_client.get_blob_client(blob_name)
    
    # Download the blob content
    blob_data = blob_client.download_blob()
    
    # Convert blob data to string and return
    return blob_data.readall().decode('utf-8')



# for when it comes to decryption
from azure.storage.blob import BlobServiceClient
from cryptography.fernet import Fernet
import pickle
import pandas as pd

def download_encrypted_pickle_to_df(
    connection_string: str, 
    container_name: str, 
    blob_name: str,
    fernet_key_string: str,
):
    # Create the BlobServiceClient object which will be used to create a container client
    blob_service_client = BlobServiceClient.from_connection_string(connection_string)
    container_client = blob_service_client.get_container_client(container_name)
    blob_client = container_client.get_blob_client(blob_name)
    blob_data_encrypted = blob_client.download_blob().readall()
    
    fernet = Fernet(fernet_key_string)
    df_as_dictionary = pickle.loads(fernet.decrypt(blob_data_encrypted))
    return pd.DataFrame(df_as_dictionary)


download_encrypted_pickle_to_df(
    connection_string, 
    container_name, 
    blob_name,
    key_string,
)
